@@ -4,6 +4,7 @@
 # ######################################### #
 
 import numpy as np
+import xtrack as xt
 from scipy.constants import c as clight
 from scipy.constants import epsilon_0
 from scipy.constants import mu_0
@@ -45,6 +46,7 @@ _factorial = np.array(
     ]
 )
 
+
 class Marker(Element):
 
     _description = []
@@ -52,6 +54,7 @@ class Marker(Element):
     def track(self, p):
         pass
 
+
 class Drift(Element):
     """Drift in expanded form"""
 
@@ -382,6 +385,216 @@ class Elens(Element):
         p.py = yp/p.rpp
 
 
+class Misalignment(Element):
+    _description = [
+        ("dx", "m", "Misalignment in x", 0),
+        ("dy", "m", "Misalignment in y", 0),
+        ("ds", "m", "Misalignment in s", 0),
+        ("theta", "rad", "Rotation around the y axis (positive s to x)", 0),
+        ("phi", "rad", "Rotation around the x axis (positive s to y)", 0),
+        ("psi", "rad", "Rotation around the s axis (positive y to x)", 0),
+        ("anchor", "", "Reference point, anchor, of the misalignment "
+                       "(0 for entry, 0.5 for middle, 1 for exit, etc.)", 0),
+        ("length", "m", "Length of the element to which the misalignment applies", 0),
+        ("angle", "rad", "Angle of bending if applicable (0 for straight)", 0),
+        ("tilt", "rad", "Tilt of the element (0 for no tilt, positive for tilt towards x)", 0),
+        ("is_exit", "", "Whether we are at the exit of the element", False),
+    ]
+
+    def track(self, particles):
+        if not self.is_exit:
+            if self.angle:
+                self.track_bent_entry(particles)
+            else:
+                self.track_straight_entry(particles)
+        else:
+            if self.angle:
+                self.track_bent_exit(particles)
+            else:
+                self.track_straight_exit(particles)
+
+    def track_bent_entry(self, particles):
+        dx, dy, ds = self.dx, self.dy, self.ds
+        theta, phi, psi = self.theta, self.phi, self.psi
+        f, length, angle, tilt = self.anchor, self.length, self.angle, self.tilt
+
+        s_phi, c_phi = np.sin(phi), np.cos(phi)
+        s_theta, c_theta = np.sin(theta), np.cos(theta)
+        s_psi, c_psi = np.sin(psi), np.cos(psi)
+        misalignment_matrix = np.array([
+            [-s_phi * s_psi * s_theta + c_psi * c_theta, -c_psi * s_phi * s_theta - c_theta * s_psi, c_phi * s_theta, dx],
+            [c_phi * s_psi, c_phi * c_psi, s_phi, dy],
+            [-c_theta * s_phi * s_psi - c_psi * s_theta, -c_psi * c_theta * s_phi + s_psi * s_theta, c_phi * c_theta, ds],
+            [0, 0, 0, 1],
+        ])
+
+        rho = length / angle
+        matrix_first_half = np.array([
+            [
+                (np.cos(f * angle) - 1) * np.cos(tilt) ** 2 + 1,
+                (np.cos(f * angle) - 1) * np.cos(tilt) * np.sin(tilt),
+                -np.cos(tilt) * np.sin(f * angle),
+                rho * (np.cos(f * angle) - 1) * np.cos(tilt),
+            ],
+            [
+                (np.cos(f * angle) - 1) * np.cos(tilt) * np.sin(tilt),
+                (np.cos(f * angle) - 1) * np.sin(tilt) ** 2 + 1,
+                -np.sin(f * angle) * np.sin(tilt),
+                rho * (np.cos(f * angle) - 1) * np.sin(tilt),
+            ],
+            [
+                np.cos(tilt) * np.sin(f * angle),
+                np.sin(f * angle) * np.sin(tilt),
+                np.cos(f * angle),
+                rho * np.sin(f * angle),
+            ],
+            [0, 0, 0, 1],
+        ])
+
+        inv_matrix_first_half = np.linalg.inv(matrix_first_half)
+
+        temp = matrix_first_half @ misalignment_matrix
+
+        misaligned_entry = temp @ inv_matrix_first_half
+
+        mis_x, mis_y, mis_s = misaligned_entry[:3, 3]
+        rot_theta = np.arctan2(misaligned_entry[0, 2], misaligned_entry[2, 2])
+        rot_phi = np.arctan2(misaligned_entry[1, 2], np.sqrt(misaligned_entry[1, 0] ** 2 + misaligned_entry[1, 1] ** 2))
+        rot_psi = np.arctan2(misaligned_entry[1, 0], misaligned_entry[1, 1])
+
+        line = xt.Line(
+            elements=[
+                xt.XYShift(dx=mis_x, dy=mis_y),
+                xt.DriftExact(length=mis_s),
+                xt.YRotation(angle=np.rad2deg(rot_theta)),
+                xt.XRotation(angle=np.rad2deg(-rot_phi)),  # flipped angle convention
+                xt.SRotation(angle=np.rad2deg(rot_psi)),
+            ]
+        )
+
+        particles.zeta -= mis_s
+        particles.s -= mis_s
+        line.track(particles)
+
+    def track_straight_entry(self, particles):
+        dx, dy, ds = self.dx, self.dy, self.ds
+        theta, phi, psi = self.theta, self.phi, self.psi
+        f, length, angle = self.anchor, self.length, self.angle
+
+        mis_x = dx - f * length * np.cos(phi) * np.sin(theta)
+        mis_y = dy - f * length * np.sin(phi)
+        mis_s = ds - f * length * (np.cos(phi) * np.cos(theta) - 1)
+
+        line = xt.Line(
+            elements=[
+                xt.XYShift(dx=mis_x, dy=mis_y),
+                xt.Solenoid(length=mis_s),
+                xt.YRotation(angle=np.rad2deg(theta)),
+                xt.XRotation(angle=np.rad2deg(-phi)),  # flipped angle convention
+                xt.SRotation(angle=np.rad2deg(psi)),
+            ]
+        )
+
+        particles.zeta -= mis_s
+        particles.s -= mis_s
+        line.track(particles)
+
+    def track_bent_exit(self, particles):
+        dx, dy, ds = self.dx, self.dy, self.ds
+        theta, phi, psi = self.theta, self.phi, self.psi
+        f, length, angle, tilt = self.anchor, self.length, self.angle, self.tilt
+
+        s_phi, c_phi = np.sin(phi), np.cos(phi)
+        s_theta, c_theta = np.sin(theta), np.cos(theta)
+        s_psi, c_psi = np.sin(psi), np.cos(psi)
+        misalignment_matrix = np.array([
+            [-s_phi * s_psi * s_theta + c_psi * c_theta, -c_psi * s_phi * s_theta - c_theta * s_psi, c_phi * s_theta, dx],
+            [c_phi * s_psi, c_phi * c_psi, s_phi, dy],
+            [-c_theta * s_phi * s_psi - c_psi * s_theta, -c_psi * c_theta * s_phi + s_psi * s_theta, c_phi * c_theta, ds],
+            [0, 0, 0, 1],
+        ])
+
+        m00, m01, m02, m03 = misalignment_matrix[0, :]
+        m10, m11, m12, m13 = misalignment_matrix[1, :]
+        m20, m21, m22, m23 = misalignment_matrix[2, :]
+        inv_misalignment_matrix = np.array([
+            [m00, m10, m20, -m00 * m03 - m10 * m13 - m20 * m23],
+            [m01, m11, m21, -m01 * m03 - m11 * m13 - m21 * m23],
+            [m02, m12, m22, -m02 * m03 - m12 * m13 - m22 * m23],
+            [0, 0, 0, 1],
+        ])
+
+        f_compl = 1 - f
+        rho = length / angle
+        matrix_second_half = np.array([
+            [
+                (np.cos(f_compl * angle) - 1) * np.cos(tilt) ** 2 + 1,
+                (np.cos(f_compl * angle) - 1) * np.cos(tilt) * np.sin(tilt),
+                np.cos(tilt) * -np.sin(f_compl * angle),
+                rho * (np.cos(f_compl * angle) - 1) * np.cos(tilt),
+            ],
+            [
+                (np.cos(f_compl * angle) - 1) * np.cos(tilt) * np.sin(tilt),
+                (np.cos(f_compl * angle) - 1) * np.sin(tilt) ** 2 + 1,
+                -np.sin(f_compl * angle) * np.sin(tilt),
+                rho * (np.cos(f_compl * angle) - 1) * np.sin(tilt),
+            ],
+            [
+                -np.cos(tilt) * -np.sin(f_compl * angle),
+                np.sin(f_compl * angle) * np.sin(tilt),
+                np.cos(f_compl * angle),
+                rho * np.sin(f_compl * angle),
+            ],
+            [0, 0, 0, 1],
+        ])
+
+        inv_matrix_second_half = np.linalg.inv(matrix_second_half)
+
+        realign = inv_matrix_second_half @ inv_misalignment_matrix @ matrix_second_half
+
+        mis_x, mis_y, mis_s = realign[:3, 3]
+        rot_theta = np.arctan2(realign[0, 2], realign[2, 2])
+        rot_phi = np.arctan2(realign[1, 2], np.sqrt(realign[1, 0] ** 2 + realign[1, 1] ** 2))
+        rot_psi = np.arctan2(realign[1, 0], realign[1, 1])
+
+        line = xt.Line(
+            elements=[
+                xt.XYShift(dx=mis_x, dy=mis_y),
+                xt.Solenoid(length=mis_s),
+                xt.YRotation(angle=np.rad2deg(rot_theta)),
+                xt.XRotation(angle=np.rad2deg(-rot_phi)),  # flipped angle convention
+                xt.SRotation(angle=np.rad2deg(rot_psi)),
+            ]
+        )
+
+        particles.zeta -= mis_s
+        particles.s -= mis_s
+        line.track(particles)
+
+    def track_straight_exit(self, particles):
+        dx, dy, ds = self.dx, self.dy, self.ds
+        theta, phi, psi = self.theta, self.phi, self.psi
+        f, length, angle = self.anchor, self.length, self.angle
+
+        mis_x = (f - 1) * length * np.cos(phi) * np.sin(theta) - dx
+        mis_y = (f - 1) * length * np.sin(phi) - dy
+        mis_s = (f - 1) * length * (np.cos(phi) * np.cos(theta) - 1) - ds
+
+        line = xt.Line(
+            elements=[
+                xt.SRotation(angle=np.rad2deg(-psi)),
+                xt.XRotation(angle=np.rad2deg(phi)),  # flipped angle convention
+                xt.YRotation(angle=np.rad2deg(-theta)),
+                xt.Solenoid(length=mis_s),
+                xt.XYShift(dx=mis_x, dy=mis_y),
+            ]
+        )
+
+        particles.zeta -= mis_s
+        particles.s -= mis_s
+        line.track(particles)
+
+
 class Wire(Element):
     """Current-carrying wire"""
 
@@ -634,6 +847,7 @@ class LinearTransferMatrix(Element):
         ("Q_y","","",0.0),
         ("beta_s","","",0.0),
         ("Q_s","","",0.0),
+        ("bucket_length","","",0.0),
         ("chroma_x","","",0.0),
         ("chroma_y","","",0.0),
         ("det_xx","","",0.0),
@@ -651,14 +865,18 @@ class LinearTransferMatrix(Element):
         ("y_ref_1","","",0.0),
         ("py_ref_1","","",0.0),
         ("damping_rate_x","","",0.0),
+        ("damping_rate_px","","",0.0),
         ("damping_rate_y","","",0.0),
-        ("damping_rate_s","","",0.0),
-        ("equ_emit_x","","",0.0),
-        ("equ_emit_y","","",0.0),
-        ("equ_emit_s","","",0.0),
+        ("damping_rate_py","","",0.0),
+        ("damping_rate_zeta","","",0.0),
+        ("damping_rate_pzeta","","",0.0),
         ("gauss_noise_ampl_x","","",0.0),
+        ("gauss_noise_ampl_px","","",0.0),
         ("gauss_noise_ampl_y","","",0.0),
-        ("gauss_noise_ampl_s","","",0.0)]
+        ("gauss_noise_ampl_py","","",0.0),
+        ("gauss_noise_ampl_zeta","","",0.0),
+        ("gauss_noise_ampl_pzeta","","",0.0),
+        ("damping_matrix","","",0.0),]
 
     def track(self,p):
         sin = p._m.sin
@@ -717,10 +935,16 @@ class LinearTransferMatrix(Element):
         p.x,p.px = M00_x*p.x + M01_x*p.px, M10_x*p.x + M11_x*p.px
         p.y,p.py = M00_y*p.y + M01_y*p.py, M10_y*p.y + M11_y*p.py
 
+        shift = 0.0
+        if self.bucket_length > 0:
+            bucket_length_zeta = self.bucket_length*p.beta0*clight
+            shift = bucket_length_zeta*np.floor(p.zeta/bucket_length_zeta+0.5);
+            p.zeta -= shift
+            
         pzeta_new = sin_s*p.zeta/self.beta_s+cos_s*p.pzeta
         zeta_new = cos_s*p.zeta-self.beta_s*sin_s*p.pzeta
 
-        p.zeta = zeta_new
+        p.zeta = zeta_new + shift
         p.pzeta = pzeta_new
 
         if self.energy_increment !=0:
@@ -746,37 +970,50 @@ class LinearTransferMatrix(Element):
             p.y *= geo_emit_factor
             p.py = old_py * geo_emit_factor
 
-        if self.damping_rate_x < 0.0 or self.damping_rate_y < 0.0 or self.damping_rate_s < 0.0:
-            raise ValueError("Damping rates cannot be negative")
-        if self.equ_emit_x < 0.0 or self.equ_emit_y < 0.0 or self.equ_emit_s < 0.0:
-            raise ValueError("Equilibrium emittance cannot be negative")
-        if self.damping_rate_x > 0.0:
-            factor = 1.0-self.damping_rate_x/2 
-            p.x *= factor
-            p.px *= factor 
-            if self.equ_emit_x > 0.0:
-                p.px += np.sqrt(2.0*self.equ_emit_x*self.damping_rate_x/self.beta_x_1)*np.random.randn(len(p.px))
-        if self.damping_rate_y > 0.0:
-            factor = 1.0-self.damping_rate_y/2 
-            p.y *= factor
-            p.py *= factor 
-            if self.equ_emit_y > 0.0:
-                p.py += np.sqrt(2.0*self.equ_emit_y*self.damping_rate_y/self.beta_y_1)*np.random.randn(len(p.py))
-        if self.damping_rate_s > 0.0:
-            factor = 1.0-self.damping_rate_s/2
-            p.zeta *= factor
-            p.delta *= factor
-            if self.equ_emit_s > 0.0:
-                p.delta += np.sqrt(2.0*self.equ_emit_s*self.damping_rate_s/self.beta_s)*np.random.randn(len(p.delta))
+        if not hasattr(self.damping_matrix,'__iter__'):
+            if (self.damping_rate_x < 0.0 or self.damping_rate_px < 0.0
+                    or self.damping_rate_y < 0.0 or self.damping_rate_py < 0.0
+                    or self.damping_rate_zeta < 0.0 or self.damping_rate_pzeta < 0.0):
+                raise ValueError("Damping rates cannot be negative")
+            if self.damping_rate_x > 0.0:
+                p.x *= 1.0-self.damping_rate_x
+            if self.damping_rate_px > 0.0:
+                p.px *= 1.0-self.damping_rate_px
+            if self.damping_rate_y > 0.0:
+                p.y *= 1.0-self.damping_rate_y
+            if self.damping_rate_py > 0.0:
+                p.py *= 1.0-self.damping_rate_py
+            if self.damping_rate_zeta > 0.0:
+                p.zeta *= 1.0-self.damping_rate_zeta
+            if self.damping_rate_pzeta > 0.0:
+                p.pzeta *= 1.0-self.damping_rate_pzeta
+        else:
+            assert np.shape(self.damping_matrix) == (6,6)
+            transformation = self.damping_matrix + np.identity(6)
+            p.x,p.px,p.y,p.py,p.zeta,p.pzeta = transformation[0,0]*p.x+transformation[0,1]*p.px+transformation[0,2]*p.y+transformation[0,3]*p.py+transformation[0,4]*p.zeta+transformation[0,5]*p.pzeta,\
+                                               transformation[1,0]*p.x+transformation[1,1]*p.px+transformation[1,2]*p.y+transformation[1,3]*p.py+transformation[1,4]*p.zeta+transformation[1,5]*p.pzeta,\
+                                               transformation[2,0]*p.x+transformation[2,1]*p.px+transformation[2,2]*p.y+transformation[2,3]*p.py+transformation[2,4]*p.zeta+transformation[2,5]*p.pzeta,\
+                                               transformation[3,0]*p.x+transformation[3,1]*p.px+transformation[3,2]*p.y+transformation[3,3]*p.py+transformation[3,4]*p.zeta+transformation[3,5]*p.pzeta,\
+                                               transformation[4,0]*p.x+transformation[4,1]*p.px+transformation[4,2]*p.y+transformation[4,3]*p.py+transformation[4,4]*p.zeta+transformation[4,5]*p.pzeta,\
+                                               transformation[5,0]*p.x+transformation[5,1]*p.px+transformation[5,2]*p.y+transformation[5,3]*p.py+transformation[5,4]*p.zeta+transformation[5,5]*p.pzeta
 
-        if self.gauss_noise_ampl_x < 0.0 or self.gauss_noise_ampl_y < 0.0 or self.gauss_noise_ampl_s < 0.0:
+
+        if (self.gauss_noise_ampl_x < 0.0 or self.gauss_noise_ampl_px < 0.0 
+                or self.gauss_noise_ampl_y < 0.0 or self.gauss_noise_ampl_py < 0.0
+                or self.gauss_noise_ampl_zeta < 0.0 or self.gauss_noise_ampl_pzeta < 0.0):
             raise ValueError("Noise amplitude cannot be negative")
+        if self.gauss_noise_ampl_x > 0.0:
+            p.x += self.gauss_noise_ampl_x*np.random.randn(len(p.x))
         if self.gauss_noise_ampl_x > 0.0:
             p.px += self.gauss_noise_ampl_x*np.random.randn(len(p.px))
         if self.gauss_noise_ampl_y > 0.0:
-            p.py += self.gauss_noise_ampl_y*np.random.randn(len(p.py))
-        if self.gauss_noise_ampl_s > 0.0:
-            p.delta += self.gauss_noise_ampl_s*np.random.randn(len(p.delta))
+            p.y += self.gauss_noise_ampl_y*np.random.randn(len(p.y))
+        if self.gauss_noise_ampl_py > 0.0:
+            p.py += self.gauss_noise_ampl_py*np.random.randn(len(p.py))
+        if self.gauss_noise_ampl_zeta > 0.0:
+            p.zeta += self.gauss_noise_ampl_zeta*np.random.randn(len(p.zeta))
+        if self.gauss_noise_ampl_pzeta > 0.0:
+            p.pzeta += self.gauss_noise_ampl_pzeta*np.random.randn(len(p.pzeta))
 
         # re-adding dispersion and closed orbit
         old_x=p.x
