@@ -9,6 +9,8 @@ import xobjects as xo
 import xtrack as xt
 import xpart as xp
 from xobjects.test_helpers import for_all_test_contexts
+import pytest
+from cpymad.madx import Madx
 
 
 @for_all_test_contexts
@@ -116,13 +118,10 @@ def test_aperture_polygon(test_context):
     xo.assert_allclose(ctx2np(parttest.state), 0)
 
 
-def test_mad_import():
+@pytest.mark.parametrize("loader", ['cpymad', 'native'])
+def test_mad_import(loader):
 
-    from cpymad.madx import Madx
-
-    mad = Madx(stdout=False)
-
-    mad.input("""
+    mad_src = """
         m_circle: marker, apertype="circle", aperture={.2};
         m_ellipse: marker, apertype="ellipse", aperture={.2, .1};
         m_rectangle: marker, apertype="rectangle", aperture={.07, .05};
@@ -140,14 +139,16 @@ def test_mad_import():
             m_octagon, at=0.05;
             m_polygon, at=0.06;
         endsequence;
-
-        use,sequence=ss;
-        twiss,betx=1,bety=1;
         """
-        )
-
-    line = xt.Line.from_madx_sequence(mad.sequence.ss, install_apertures=True)
-
+    if loader == 'cpymad':
+        mad = Madx()
+        mad.input(mad_src)
+        mad.beam()
+        mad.use('ss')
+        line = xt.Line.from_madx_sequence(mad.sequence.ss, install_apertures=True)
+    elif loader == 'native':
+        env = xt.load(string=mad_src, format='madx')
+        line = env.ss
     apertures = [ee for ee in line.elements if ee.__class__.__name__.startswith('Limit')]
 
     circ = apertures[0]
@@ -210,10 +211,15 @@ def test_mad_import():
     assert polyg.__class__.__name__ == 'LimitPolygon'
     assert len(polyg._xobject.x_vertices) == 3
     assert len(polyg._xobject.y_vertices) == 3
-    assert polyg._xobject.x_vertices[0] == -8.8e-2
-    assert polyg._xobject.y_vertices[0] == 0
-    assert polyg._xobject.x_vertices[1] == 5.8e-2
-    assert polyg._xobject.y_vertices[1] == -3.5e-2
+    # assert polyg._xobject.x_vertices[0] == -8.8e-2
+    # assert polyg._xobject.y_vertices[0] == 0
+    # assert polyg._xobject.x_vertices[1] == 5.8e-2
+    # assert polyg._xobject.y_vertices[1] == -3.5e-2
+    # breakpoint()
+    xo.assert_allclose(polyg.x_vertices, np.array([+5.800e-2,+5.800e-2,-8.800e-2]),
+                       atol=1e-14, rtol=0)
+    xo.assert_allclose(polyg.y_vertices, np.array([+3.500e-2,-3.500e-2,+0.000e+0]),
+                          atol=1e-14, rtol=0)
 
 
 @for_all_test_contexts
@@ -301,62 +307,64 @@ def test_aperture_svg_path():
     aper2=xt.LimitPolygon.from_dict(aper.to_dict())
     assert aper2.svg==aper.svg
 
-    x_expected = np.array([ 3.41600000e-03,  3.15400000e-03,  2.92800000e-03,  2.75000000e-03,
-        2.63200000e-03,  2.58600000e-03,  2.62400000e-03,  2.75800000e-03,
-        3.00000000e-03,  2.40800000e-03,  2.12700000e-03,  1.86400000e-03,
-        1.62500000e-03,  1.41600000e-03,  1.24300000e-03,  1.11200000e-03,
-        1.02900000e-03,  1.00000000e-03,  7.24296143e-04,  5.96239745e-04,
-        4.76593043e-04,  3.66655177e-04,  2.67619865e-04,  1.80562445e-04,
-        1.06428199e-04,  4.60220848e-05, -4.44089210e-19,  7.05663559e-04,
-        1.04481682e-03,  1.37237342e-03,  1.68655149e-03,  1.98564190e-03,
-        2.26801764e-03,  2.53214260e-03,  2.77657996e-03,  3.00000000e-03,
-        3.01200000e-03,  2.99700000e-03,  2.96800000e-03,  2.92500000e-03,
-        2.86800000e-03,  2.79700000e-03,  2.71200000e-03,  2.61300000e-03,
-        2.50000000e-03,  2.25600000e-03,  2.13100000e-03,  2.00400000e-03,
-        1.87500000e-03,  1.74400000e-03,  1.61100000e-03,  1.47600000e-03,
-        1.33900000e-03,  1.20000000e-03,  1.00800000e-03,  9.78000000e-04,
-        9.92000000e-04,  1.05000000e-03,  1.15200000e-03,  1.29800000e-03,
-        1.48800000e-03,  1.72200000e-03,  2.00000000e-03,  2.54000000e-03,
-        2.76500000e-03,  2.96000000e-03,  3.12500000e-03,  3.26000000e-03,
-        3.36500000e-03,  3.44000000e-03,  3.48500000e-03,  3.50000000e-03,
-        3.52000000e-03,  3.54500000e-03,  3.58000000e-03,  3.62500000e-03,
-        3.68000000e-03,  3.74500000e-03,  3.82000000e-03,  3.90500000e-03,
-        4.00000000e-03,  4.56952827e-03,  4.82689224e-03,  5.06408867e-03,
-        5.27982723e-03,  5.47293432e-03,  5.64235946e-03,  5.78718097e-03,
-        5.90661105e-03,  6.00000000e-03,  6.27570386e-03,  6.40376026e-03,
-        6.52340696e-03,  6.63334482e-03,  6.73238014e-03,  6.81943755e-03,
-        6.89357180e-03,  6.95397792e-03,  7.00000000e-03,  7.38400000e-03,
-        7.44100000e-03,  7.43200000e-03,  7.37500000e-03,  7.28800000e-03,
-        7.18900000e-03,  7.09600000e-03,  7.02700000e-03,  7.00000000e-03,
-        6.50400000e-03,  6.31600000e-03,  6.15200000e-03,  6.00000000e-03,
-        5.84800000e-03,  5.68400000e-03,  5.49600000e-03,  5.27200000e-03,
-        5.00000000e-03,  1.20000000e-02,  1.30000000e-02,  1.00000000e-02,
-        4.00000000e-03])
-    y_expected = np.array([-0.007504  , -0.007316  , -0.007152  , -0.007     , -0.006848  ,
-       -0.006684  , -0.006496  , -0.006272  , -0.006     , -0.005904  ,
-       -0.005811  , -0.005712  , -0.005625  , -0.005568  , -0.005559  ,
-       -0.005616  , -0.005757  , -0.006     , -0.00610643, -0.00618056,
-       -0.00626762, -0.00636666, -0.00647659, -0.00659624, -0.0067243 ,
-       -0.00685937, -0.007     , -0.00721282, -0.00735764, -0.00752707,
-       -0.00772017, -0.00793591, -0.00817311, -0.00843047, -0.0087066 ,
-       -0.009     , -0.00918   , -0.009255  , -0.00932   , -0.009375  ,
-       -0.00942   , -0.009455  , -0.00948   , -0.009495  , -0.0095    ,
-       -0.00956   , -0.009635  , -0.00974   , -0.009875  , -0.01004   ,
-       -0.010235  , -0.01046   , -0.010715  , -0.011     , -0.011512  ,
-       -0.011702  , -0.011848  , -0.01195   , -0.012008  , -0.012022  ,
-       -0.011992  , -0.011918  , -0.0118    , -0.011524  , -0.011389  ,
-       -0.011256  , -0.011125  , -0.010996  , -0.010869  , -0.010744  ,
-       -0.010621  , -0.0105    , -0.010288  , -0.010203  , -0.010132  ,
-       -0.010075  , -0.010032  , -0.010003  , -0.009988  , -0.009987  ,
-       -0.01      , -0.01046786, -0.01073198, -0.01101436, -0.01131345,
-       -0.01162763, -0.01195518, -0.01229434, -0.01264324, -0.013     ,
-       -0.01289357, -0.01281944, -0.01273238, -0.01263334, -0.01252341,
-       -0.01240376, -0.0122757 , -0.01214063, -0.012     , -0.011888  ,
-       -0.011757  , -0.011584  , -0.011375  , -0.011136  , -0.010873  ,
-       -0.010592  , -0.010299  , -0.01      , -0.010376  , -0.010414  ,
-       -0.010368  , -0.01025   , -0.010072  , -0.009846  , -0.009584  ,
-       -0.009298  , -0.009     , -0.003     , -0.        , -0.001     ,
-       -0.008     ])
+    x_expected = np.array([
+         4.00000000e-03,  1.00000000e-02,  1.30000000e-02,  1.20000000e-02,
+         5.00000000e-03,  5.27200000e-03,  5.49600000e-03,  5.68400000e-03,
+         5.84800000e-03,  6.00000000e-03,  6.15200000e-03,  6.31600000e-03,
+         6.50400000e-03,  7.00000000e-03,  7.02700000e-03,  7.09600000e-03,
+         7.18900000e-03,  7.28800000e-03,  7.37500000e-03,  7.43200000e-03,
+         7.44100000e-03,  7.38400000e-03,  7.00000000e-03,  6.95397792e-03,
+         6.89357180e-03,  6.81943755e-03,  6.73238014e-03,  6.63334482e-03,
+         6.52340696e-03,  6.40376026e-03,  6.27570386e-03,  6.00000000e-03,
+         5.90661105e-03,  5.78718097e-03,  5.64235946e-03,  5.47293432e-03,
+         5.27982723e-03,  5.06408867e-03,  4.82689224e-03,  4.56952827e-03,
+         4.00000000e-03,  3.90500000e-03,  3.82000000e-03,  3.74500000e-03,
+         3.68000000e-03,  3.62500000e-03,  3.58000000e-03,  3.54500000e-03,
+         3.52000000e-03,  3.50000000e-03,  3.48500000e-03,  3.44000000e-03,
+         3.36500000e-03,  3.26000000e-03,  3.12500000e-03,  2.96000000e-03,
+         2.76500000e-03,  2.54000000e-03,  2.00000000e-03,  1.72200000e-03,
+         1.48800000e-03,  1.29800000e-03,  1.15200000e-03,  1.05000000e-03,
+         9.92000000e-04,  9.78000000e-04,  1.00800000e-03,  1.20000000e-03,
+         1.33900000e-03,  1.47600000e-03,  1.61100000e-03,  1.74400000e-03,
+         1.87500000e-03,  2.00400000e-03,  2.13100000e-03,  2.25600000e-03,
+         2.50000000e-03,  2.61300000e-03,  2.71200000e-03,  2.79700000e-03,
+         2.86800000e-03,  2.92500000e-03,  2.96800000e-03,  2.99700000e-03,
+         3.01200000e-03,  3.00000000e-03,  2.77657996e-03,  2.53214260e-03,
+         2.26801764e-03,  1.98564190e-03,  1.68655149e-03,  1.37237342e-03,
+         1.04481682e-03,  7.05663559e-04, -4.44089210e-19,  4.60220848e-05,
+         1.06428199e-04,  1.80562445e-04,  2.67619865e-04,  3.66655177e-04,
+         4.76593043e-04,  5.96239745e-04,  7.24296143e-04,  1.00000000e-03,
+         1.02900000e-03,  1.11200000e-03,  1.24300000e-03,  1.41600000e-03,
+         1.62500000e-03,  1.86400000e-03,  2.12700000e-03,  2.40800000e-03,
+         3.00000000e-03,  2.75800000e-03,  2.62400000e-03,  2.58600000e-03,
+         2.63200000e-03,  2.75000000e-03,  2.92800000e-03,  3.15400000e-03,
+         3.41600000e-03])
+    y_expected = np.array([
+        -0.008     , -0.001     , -0.        , -0.003     , -0.009     ,
+       -0.009298  , -0.009584  , -0.009846  , -0.010072  , -0.01025   ,
+       -0.010368  , -0.010414  , -0.010376  , -0.01      , -0.010299  ,
+       -0.010592  , -0.010873  , -0.011136  , -0.011375  , -0.011584  ,
+       -0.011757  , -0.011888  , -0.012     , -0.01214063, -0.0122757 ,
+       -0.01240376, -0.01252341, -0.01263334, -0.01273238, -0.01281944,
+       -0.01289357, -0.013     , -0.01264324, -0.01229434, -0.01195518,
+       -0.01162763, -0.01131345, -0.01101436, -0.01073198, -0.01046786,
+       -0.01      , -0.009987  , -0.009988  , -0.010003  , -0.010032  ,
+       -0.010075  , -0.010132  , -0.010203  , -0.010288  , -0.0105    ,
+       -0.010621  , -0.010744  , -0.010869  , -0.010996  , -0.011125  ,
+       -0.011256  , -0.011389  , -0.011524  , -0.0118    , -0.011918  ,
+       -0.011992  , -0.012022  , -0.012008  , -0.01195   , -0.011848  ,
+       -0.011702  , -0.011512  , -0.011     , -0.010715  , -0.01046   ,
+       -0.010235  , -0.01004   , -0.009875  , -0.00974   , -0.009635  ,
+       -0.00956   , -0.0095    , -0.009495  , -0.00948   , -0.009455  ,
+       -0.00942   , -0.009375  , -0.00932   , -0.009255  , -0.00918   ,
+       -0.009     , -0.0087066 , -0.00843047, -0.00817311, -0.00793591,
+       -0.00772017, -0.00752707, -0.00735764, -0.00721282, -0.007     ,
+       -0.00685937, -0.0067243 , -0.00659624, -0.00647659, -0.00636666,
+       -0.00626762, -0.00618056, -0.00610643, -0.006     , -0.005757  ,
+       -0.005616  , -0.005559  , -0.005568  , -0.005625  , -0.005712  ,
+       -0.005811  , -0.005904  , -0.006     , -0.006272  , -0.006496  ,
+       -0.006684  , -0.006848  , -0.007     , -0.007152  , -0.007316  ,
+       -0.007504  ])
 
     xo.assert_allclose(aper.x_vertices, x_expected, atol=1e-6, rtol=0)
     xo.assert_allclose(aper.y_vertices, y_expected, atol=1e-6, rtol=0)
@@ -378,4 +386,4 @@ def test_limitrect_to_dict():
     assert lrectdef.min_x == -1e10
     assert lrectdef.max_x == 1e10
     assert lrectdef.min_y == -1e10
-    assert lrectdef.max_y == 1e10
\ No newline at end of file
+    assert lrectdef.max_y == 1e10
