@@ -94,63 +94,63 @@ def _make_line_with_expressions():
 
     line.vars['length_drift'] = 999.
 
-    line.element_refs['e0'].k0 = line.vars['k0_bend']
-    line.element_refs['e0'].angle = line.vars['angle_bend']
-    line.element_refs['e0'].length = line.vars['length_bend']
-    line.element_refs['e0'].shift_x = line.vars['shift_x_bend']
-    line.element_refs['e0'].shift_y = line.vars['shift_y_bend']
-    line.element_refs['e0'].shift_s = line.vars['shift_s_bend']
-    line.element_refs['e0'].rot_s_rad = line.vars['rot_s_rad_bend']
-    line.element_refs['e0'].k1 = line.vars['k1_bend']
-    line.element_refs['e0'].knl[0] = line.vars['knl_bend_0']
-    line.element_refs['e0'].knl[1] = line.vars['knl_bend_1']
-    line.element_refs['e0'].knl[2] = line.vars['knl_bend_2']
-    line.element_refs['e0'].knl[3] = line.vars['knl_bend_3']
-    line.element_refs['e0'].ksl[0] = line.vars['ksl_bend_0']
-    line.element_refs['e0'].ksl[1] = line.vars['ksl_bend_1']
-    line.element_refs['e0'].ksl[2] = line.vars['ksl_bend_2']
-    line.element_refs['e0'].ksl[3] = line.vars['ksl_bend_3']
+    line['e0'].k0 = line.vars['k0_bend']
+    line['e0'].angle = line.vars['angle_bend']
+    line['e0'].length = line.vars['length_bend']
+    line['e0'].shift_x = line.vars['shift_x_bend']
+    line['e0'].shift_y = line.vars['shift_y_bend']
+    line['e0'].shift_s = line.vars['shift_s_bend']
+    line['e0'].rot_s_rad = line.vars['rot_s_rad_bend']
+    line['e0'].k1 = line.vars['k1_bend']
+    line['e0'].knl[0] = line.vars['knl_bend_0']
+    line['e0'].knl[1] = line.vars['knl_bend_1']
+    line['e0'].knl[2] = line.vars['knl_bend_2']
+    line['e0'].knl[3] = line.vars['knl_bend_3']
+    line['e0'].ksl[0] = line.vars['ksl_bend_0']
+    line['e0'].ksl[1] = line.vars['ksl_bend_1']
+    line['e0'].ksl[2] = line.vars['ksl_bend_2']
+    line['e0'].ksl[3] = line.vars['ksl_bend_3']
 
-    line.element_refs['e1'].k1 = line.vars['k1_quad']
-    line.element_refs['e1'].k1s = line.vars['k1s_quad']
-    line.element_refs['e1'].length = line.vars['length_quad']
-    line.element_refs['e1'].shift_x = line.vars['shift_x_quad']
-    line.element_refs['e1'].shift_y = line.vars['shift_y_quad']
-    line.element_refs['e1'].shift_s = line.vars['shift_s_quad']
-    line.element_refs['e1'].rot_s_rad = line.vars['rot_s_rad_quad']
+    line['e1'].k1 = line.vars['k1_quad']
+    line['e1'].k1s = line.vars['k1s_quad']
+    line['e1'].length = line.vars['length_quad']
+    line['e1'].shift_x = line.vars['shift_x_quad']
+    line['e1'].shift_y = line.vars['shift_y_quad']
+    line['e1'].shift_s = line.vars['shift_s_quad']
+    line['e1'].rot_s_rad = line.vars['rot_s_rad_quad']
 
-    line.element_refs['e2'].k2 = line.vars['k2_sext']
-    line.element_refs['e2'].k2s = line.vars['k2s_sext']
-    line.element_refs['e2'].length = line.vars['length_sext']
-    line.element_refs['e2'].shift_x = line.vars['shift_x_sext']
-    line.element_refs['e2'].shift_y = line.vars['shift_y_sext']
-    line.element_refs['e2'].shift_s = line.vars['shift_s_sext']
-    line.element_refs['e2'].rot_s_rad = line.vars['rot_s_rad_sext']
+    line['e2'].k2 = line.vars['k2_sext']
+    line['e2'].k2s = line.vars['k2s_sext']
+    line['e2'].length = line.vars['length_sext']
+    line['e2'].shift_x = line.vars['shift_x_sext']
+    line['e2'].shift_y = line.vars['shift_y_sext']
+    line['e2'].shift_s = line.vars['shift_s_sext']
+    line['e2'].rot_s_rad = line.vars['rot_s_rad_sext']
 
-    line.element_refs['e3'].k3 = line.vars['k3_octu']
-    line.element_refs['e3'].k3s = line.vars['k3s_octu']
-    line.element_refs['e3'].length = line.vars['length_octu']
-    line.element_refs['e3'].shift_x = line.vars['shift_x_octu']
-    line.element_refs['e3'].shift_y = line.vars['shift_y_octu']
-    line.element_refs['e3'].shift_s = line.vars['shift_s_octu']
-    line.element_refs['e3'].rot_s_rad = line.vars['rot_s_rad_octu']
+    line['e3'].k3 = line.vars['k3_octu']
+    line['e3'].k3s = line.vars['k3s_octu']
+    line['e3'].length = line.vars['length_octu']
+    line['e3'].shift_x = line.vars['shift_x_octu']
+    line['e3'].shift_y = line.vars['shift_y_octu']
+    line['e3'].shift_s = line.vars['shift_s_octu']
+    line['e3'].rot_s_rad = line.vars['rot_s_rad_octu']
 
-    line.element_refs['e4'].knl[0] = line.vars['knl_mult_0']
-    line.element_refs['e4'].knl[1] = line.vars['knl_mult_1']
-    line.element_refs['e4'].knl[2] = line.vars['knl_mult_2']
-    line.element_refs['e4'].knl[3] = line.vars['knl_mult_3']
-    line.element_refs['e4'].ksl[0] = line.vars['ksl_mult_0']
-    line.element_refs['e4'].ksl[1] = line.vars['ksl_mult_1']
-    line.element_refs['e4'].ksl[2] = line.vars['ksl_mult_2']
-    line.element_refs['e4'].ksl[3] = line.vars['ksl_mult_3']
-    line.element_refs['e4'].length = line.vars['length_mult']
-    line.element_refs['e4'].shift_x = line.vars['shift_x_mult']
-    line.element_refs['e4'].shift_y = line.vars['shift_y_mult']
-    line.element_refs['e4'].shift_s = line.vars['shift_s_mult']
-    line.element_refs['e4'].rot_s_rad = line.vars['rot_s_rad_mult']
-    line.element_refs['e4'].hxl = line.vars['hxl_mult']
+    line['e4'].knl[0] = line.vars['knl_mult_0']
+    line['e4'].knl[1] = line.vars['knl_mult_1']
+    line['e4'].knl[2] = line.vars['knl_mult_2']
+    line['e4'].knl[3] = line.vars['knl_mult_3']
+    line['e4'].ksl[0] = line.vars['ksl_mult_0']
+    line['e4'].ksl[1] = line.vars['ksl_mult_1']
+    line['e4'].ksl[2] = line.vars['ksl_mult_2']
+    line['e4'].ksl[3] = line.vars['ksl_mult_3']
+    line['e4'].length = line.vars['length_mult']
+    line['e4'].shift_x = line.vars['shift_x_mult']
+    line['e4'].shift_y = line.vars['shift_y_mult']
+    line['e4'].shift_s = line.vars['shift_s_mult']
+    line['e4'].rot_s_rad = line.vars['rot_s_rad_mult']
+    line['e4'].hxl = line.vars['hxl_mult']
 
-    line.element_refs['e5'].length = line.vars['length_drift']
+    line['e5'].length = line.vars['length_drift']
 
     return line
 
