@@ -19,33 +19,16 @@ test_data_folder = pathlib.Path(
 def test_psb_chicane(test_context):
     mad = Madx(stdout=False)
 
-    # Load mad model and apply element shifts
-    mad.input(f'''
-    call, file = '{str(test_data_folder)}/psb_chicane/psb.seq';
-    call, file = '{str(test_data_folder)}/psb_chicane/psb_fb_lhc.str';
+    env = xt.load([test_data_folder / 'psb_chicane/psb.seq',
+                   test_data_folder / 'psb_chicane/psb_fb_lhc.str'])
+    line = env.psb1
+    line.set_particle_ref('proton', p0c=0.5708301551893517e9)
 
-    beam, particle=PROTON, pc=0.5708301551893517;
-    use, sequence=psb1;
+    line['bi1.bsw1l1.1'].shift_x = -0.0442
+    line['bi1.bsw1l1.2'].shift_x = -0.0442
+    line['bi1.bsw1l1.3'].shift_x = -0.0442
+    line['bi1.bsw1l1.4'].shift_x = -0.0442
 
-    select,flag=error,clear;
-    select,flag=error,pattern=bi1.bsw1l1.1*;
-    ealign, dx=-0.0057;
-
-    select,flag=error,clear;
-    select,flag=error,pattern=bi1.bsw1l1.2*;
-    select,flag=error,pattern=bi1.bsw1l1.3*;
-    select,flag=error,pattern=bi1.bsw1l1.4*;
-    ealign, dx=-0.0442;
-
-    twiss;
-    ''')
-
-    line = xt.Line.from_madx_sequence(mad.sequence.psb1,
-                                allow_thick=True,
-                                enable_align_errors=True,
-                                deferred_expressions=True)
-    line.particle_ref = xp.Particles(mass0=xp.PROTON_MASS_EV,
-                                gamma0=mad.sequence.psb1.beam.gamma)
     line.twiss_default['method'] = '4d'
     line.configure_bend_model(core='adaptive')
 
@@ -427,7 +410,7 @@ def test_psb_chicane(test_context):
 
         # Add monitor at end line
         monitor = xt.ParticlesMonitor(start_at_turn=0, stop_at_turn=10, num_particles=100)
-        line.insert_element(index='p16ring1$end', element=monitor, name='monitor_at_end')
+        line.append('monitor_at_end', monitor)
         line.build_tracker()
 
         p = line.build_particles(_capacity=100, x=0)
