@@ -5,7 +5,7 @@ from xobjects.test_helpers import for_all_test_contexts
 
 assert_allclose = xo.assert_allclose
 
-@for_all_test_contexts
+@for_all_test_contexts(excluding=('ContextCupy', 'ContextPyopencl'))
 def test_slice_thin_and_insert_with_replicas(test_context):
 
     elements = {
@@ -103,7 +103,8 @@ def test_slice_thin_and_insert_with_replicas(test_context):
         3.        , 3.        , 3.        , 3.        , 3.16666667,
         3.16666667, 3.83333333, 3.83333333, 4.        , 4.        ,
         4.        , 4.        , 4.        , 4.16666667, 4.16666667,
-        4.83333333, 4.83333333, 5.        , 5.        , 5.        ]))
+        4.83333333, 4.83333333, 5.        , 5.        , 5.        ,
+    ]), atol=1e-7)
 
     assert_allclose(tt_no_repl.s, np.array([
         0.        , 1.        , 2.        , 2.        , 2.        ,
@@ -111,7 +112,8 @@ def test_slice_thin_and_insert_with_replicas(test_context):
         3.        , 3.        , 3.        , 3.        , 3.16666667,
         3.16666667, 3.83333333, 3.83333333, 4.        , 4.        ,
         4.        , 4.        , 4.        , 4.16666667, 4.16666667,
-        4.83333333, 4.83333333, 5.        , 5.        , 5.        ]))
+        4.83333333, 4.83333333, 5.        , 5.        , 5.        ,
+    ]), atol=1e-7)
 
 
     p1 = p0.copy()
@@ -236,6 +238,8 @@ def test_slice_thick_and_insert_with_replicas(test_context):
                 element_names=list(elements.keys()))
     line.build_tracker(_context=test_context)
 
+    assert line['e2']._movable
+
     element_no_repl={
         'e0': xt.Bend(k0=0.3, h=0.31, length=1),
         'e1': xt.Bend(k0=0.3, h=0.31, length=1),
@@ -275,18 +279,26 @@ def test_slice_thick_and_insert_with_replicas(test_context):
     assert_allclose(p2.zeta, p1.zeta, rtol=0, atol=1e-14)
     assert_allclose(p2.delta, p1.delta, rtol=0, atol=1e-14)
 
+    assert line['e2']._movable
+    # line['e2']._mark = True
+
     line.slice_thick_elements(
         slicing_strategies=[
             xt.Strategy(None),
             xt.Strategy(xt.Teapot(3, mode='thick'), name='e2|e3|e4')])
+    assert line['e2']._movable
     line.build_tracker(_context=test_context)
 
+    assert line['e2']._movable
+
     line_no_repl.slice_thick_elements(
         slicing_strategies=[
             xt.Strategy(None),
             xt.Strategy(xt.Teapot(3, mode='thick'), name='e2|e3|e4')])
     line_no_repl.build_tracker(_context=test_context)
 
+    assert line['e2']._movable
+
     tt = line.get_table()
     tt_no_repl = line_no_repl.get_table()
 
@@ -318,14 +330,16 @@ def test_slice_thick_and_insert_with_replicas(test_context):
         2.16666667, 2.83333333, 3.        , 3.        , 3.        ,
         3.        , 3.        , 3.16666667, 3.83333333, 4.        ,
         4.        , 4.        , 4.        , 4.        , 4.16666667,
-        4.83333333, 5.        , 5.        , 5.        ]))
+        4.83333333, 5.        , 5.        , 5.        ,
+    ]), atol=1e-7)
 
     assert_allclose(tt_no_repl.s, np.array([
         0.        , 1.        , 2.        , 2.        , 2.        ,
         2.16666667, 2.83333333, 3.        , 3.        , 3.        ,
         3.        , 3.        , 3.16666667, 3.83333333, 4.        ,
         4.        , 4.        , 4.        , 4.        , 4.16666667,
-        4.83333333, 5.        , 5.        , 5.        ]))
+        4.83333333, 5.        , 5.        , 5.        ,
+    ]), atol=1e-7)
 
 
     p1 = p0.copy()
@@ -341,13 +355,17 @@ def test_slice_thick_and_insert_with_replicas(test_context):
     assert_allclose(p2.zeta, p1.zeta, rtol=0, atol=1e-14)
     assert_allclose(p2.delta, p1.delta, rtol=0, atol=1e-14)
 
+    assert line['e2']._movable
+
     line.discard_tracker()
+    assert line['e2']._movable
     line.insert_element(name='mkins1', element=xt.Marker(), at_s=0.5)
     line.insert_element(name='mkins2', element=xt.Marker(), at_s=1.5)
     line.insert_element(name='mkins3', element=xt.Marker(), at_s=2.5)
     line.insert_element(name='mkins4', element=xt.Marker(), at_s=3.5)
     line.insert_element(name='mkins5', element=xt.Marker(), at_s=4.5)
     line.build_tracker(_context=test_context)
+    assert line['e2']._movable
 
     line_no_repl.discard_tracker()
     line_no_repl.insert_element(name='mkins1', element=xt.Marker(), at_s=0.5)
@@ -356,6 +374,7 @@ def test_slice_thick_and_insert_with_replicas(test_context):
     line_no_repl.insert_element(name='mkins4', element=xt.Marker(), at_s=3.5)
     line_no_repl.insert_element(name='mkins5', element=xt.Marker(), at_s=4.5)
     line_no_repl.build_tracker(_context=test_context)
+    assert line['e2']._movable
 
     tt = line.get_table()
     tt_no_repl = line_no_repl.get_table()
@@ -428,6 +447,7 @@ def test_slice_thick_and_insert_with_replicas(test_context):
 
     line.track(p1)
     line_no_repl.track(p2)
+    assert line['e2']._movable
 
     assert_allclose(p2.x, p1.x, rtol=0, atol=1e-14)
     assert_allclose(p2.px, p1.px, rtol=0, atol=1e-14)
