@@ -25,13 +25,13 @@ def test_thin_slice_bend(test_context, bend_type):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..995'].parent_name == 'e0'
-    assert line['e0..995']._parent is line.element_dict['e0']
+    assert line['e0..995']._parent is line.get('e0')
     assert line['drift_e0..995'].parent_name == 'e0'
-    assert line['drift_e0..995']._parent is line.element_dict['e0']
+    assert line['drift_e0..995']._parent is line.get('e0')
     assert line['e0..entry_map'].parent_name == 'e0'
-    assert line['e0..entry_map']._parent is line.element_dict['e0']
+    assert line['e0..entry_map']._parent is line.get('e0')
     assert line['e0..exit_map'].parent_name == 'e0'
-    assert line['e0..exit_map']._parent is line.element_dict['e0']
+    assert line['e0..exit_map']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -78,16 +78,16 @@ def test_thin_slice_bend(test_context, bend_type):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..995'], thin_slice_cls)
     assert line2['e0..995'].parent_name == 'e0'
-    assert line2['e0..995']._parent is line2.element_dict['e0']
+    assert line2['e0..995']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..995'], drift_slice_type_cls)
     assert line2['drift_e0..995'].parent_name == 'e0'
-    assert line2['drift_e0..995']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..995']._parent is line2.get('e0')
     assert isinstance(line2['e0..entry_map'], slice_entry_cls)
     assert line2['e0..entry_map'].parent_name == 'e0'
-    assert line2['e0..entry_map']._parent is line2.element_dict['e0']
+    assert line2['e0..entry_map']._parent is line2.get('e0')
     assert isinstance(line2['e0..exit_map'], slice_exit_cls)
     assert line2['e0..exit_map'].parent_name == 'e0'
-    assert line2['e0..exit_map']._parent is line2.element_dict['e0']
+    assert line2['e0..exit_map']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -137,9 +137,9 @@ def test_thin_slice_quadrupole(test_context):
         slicing_strategies=[xt.Strategy(xt.Teapot(10000))])
     line.build_tracker(_context=test_context)
     assert line['e0..995'].parent_name == 'e0'
-    assert line['e0..995']._parent is line.element_dict['e0']
+    assert line['e0..995']._parent is line.get('e0')
     assert line['drift_e0..995'].parent_name == 'e0'
-    assert line['drift_e0..995']._parent is line.element_dict['e0']
+    assert line['drift_e0..995']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03,
                         _context=test_context)
@@ -167,10 +167,10 @@ def test_thin_slice_quadrupole(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..995'], xt.ThinSliceQuadrupole)
     assert line2['e0..995'].parent_name == 'e0'
-    assert line2['e0..995']._parent is line2.element_dict['e0']
+    assert line2['e0..995']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..995'], xt.DriftSliceQuadrupole)
     assert line2['drift_e0..995'].parent_name == 'e0'
-    assert line2['drift_e0..995']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..995']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -218,9 +218,9 @@ def test_thin_slice_sextupole(test_context):
         slicing_strategies=[xt.Strategy(xt.Teapot(1))])
     line.build_tracker(_context=test_context)
     assert line['e0..0'].parent_name == 'e0'
-    assert line['e0..0']._parent is line.element_dict['e0']
+    assert line['e0..0']._parent is line.get('e0')
     assert line['drift_e0..0'].parent_name == 'e0'
-    assert line['drift_e0..0']._parent is line.element_dict['e0']
+    assert line['drift_e0..0']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03,
                         _context=test_context)
@@ -248,10 +248,10 @@ def test_thin_slice_sextupole(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..0'], xt.ThinSliceSextupole)
     assert line2['e0..0'].parent_name == 'e0'
-    assert line2['e0..0']._parent is line2.element_dict['e0']
+    assert line2['e0..0']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..0'], xt.DriftSliceSextupole)
     assert line2['drift_e0..0'].parent_name == 'e0'
-    assert line2['drift_e0..0']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..0']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -297,9 +297,9 @@ def test_thin_slice_octupole(test_context):
         slicing_strategies=[xt.Strategy(xt.Teapot(1))])
     line.build_tracker(_context=test_context)
     assert line['e0..0'].parent_name == 'e0'
-    assert line['e0..0']._parent is line.element_dict['e0']
+    assert line['e0..0']._parent is line.get('e0')
     assert line['drift_e0..0'].parent_name == 'e0'
-    assert line['drift_e0..0']._parent is line.element_dict['e0']
+    assert line['drift_e0..0']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03,
                         _context=test_context)
@@ -327,10 +327,10 @@ def test_thin_slice_octupole(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..0'], xt.ThinSliceOctupole)
     assert line2['e0..0'].parent_name == 'e0'
-    assert line2['e0..0']._parent is line2.element_dict['e0']
+    assert line2['e0..0']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..0'], xt.DriftSliceOctupole)
     assert line2['drift_e0..0'].parent_name == 'e0'
-    assert line2['drift_e0..0']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..0']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -376,7 +376,7 @@ def test_thin_slice_drift(test_context):
         slicing_strategies=[xt.Strategy(xt.Uniform(5), element_type=xt.Drift)])
     line.build_tracker(_context=test_context)
     assert line['drift_e0..0'].parent_name == 'e0'
-    assert line['drift_e0..0']._parent is line.element_dict['e0']
+    assert line['drift_e0..0']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03,
                       _context=test_context)
@@ -401,7 +401,7 @@ def test_thin_slice_drift(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['drift_e0..0'], xt.DriftSlice)
     assert line2['drift_e0..0'].parent_name == 'e0'
-    assert line2['drift_e0..0']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..0']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -453,11 +453,11 @@ def test_thick_slice_bend(test_context, bend_type):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..3'].parent_name == 'e0'
-    assert line['e0..3']._parent is line.element_dict['e0']
+    assert line['e0..3']._parent is line.get('e0')
     assert line['e0..entry_map'].parent_name == 'e0'
-    assert line['e0..entry_map']._parent is line.element_dict['e0']
+    assert line['e0..entry_map']._parent is line.get('e0')
     assert line['e0..exit_map'].parent_name == 'e0'
-    assert line['e0..exit_map']._parent is line.element_dict['e0']
+    assert line['e0..exit_map']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03,
                         _context=test_context)
@@ -498,11 +498,11 @@ def test_thick_slice_bend(test_context, bend_type):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..3'], thick_slice_bend_cls)
     assert line2['e0..3'].parent_name == 'e0'
-    assert line2['e0..3']._parent is line2.element_dict['e0']
+    assert line2['e0..3']._parent is line2.get('e0')
     assert line2['e0..entry_map'].parent_name == 'e0'
-    assert line2['e0..entry_map']._parent is line2.element_dict['e0']
+    assert line2['e0..entry_map']._parent is line2.get('e0')
     assert line2['e0..exit_map'].parent_name == 'e0'
-    assert line2['e0..exit_map']._parent is line2.element_dict['e0']
+    assert line2['e0..exit_map']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -525,7 +525,7 @@ def test_thick_slice_quadrupole(test_context):
         slicing_strategies=[xt.Strategy(xt.Teapot(5, mode='thick'))])
     line.build_tracker(_context=test_context)
     assert line['e0..3'].parent_name == 'e0'
-    assert line['e0..3']._parent is line.element_dict['e0']
+    assert line['e0..3']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03,
                         _context=test_context)
@@ -551,7 +551,7 @@ def test_thick_slice_quadrupole(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..3'], xt.ThickSliceQuadrupole)
     assert line2['e0..3'].parent_name == 'e0'
-    assert line2['e0..3']._parent is line2.element_dict['e0']
+    assert line2['e0..3']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -573,7 +573,7 @@ def test_thick_slice_sextupole(test_context):
         slicing_strategies=[xt.Strategy(xt.Teapot(1, mode='thick'))])
     line.build_tracker(_context=test_context)
     assert line['e0..0'].parent_name == 'e0'
-    assert line['e0..0']._parent is line.element_dict['e0']
+    assert line['e0..0']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03,
                         _context=test_context)
@@ -599,7 +599,7 @@ def test_thick_slice_sextupole(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..0'], xt.ThickSliceSextupole)
     assert line2['e0..0'].parent_name == 'e0'
-    assert line2['e0..0']._parent is line2.element_dict['e0']
+    assert line2['e0..0']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -621,7 +621,7 @@ def test_thick_slice_octupole(test_context):
         slicing_strategies=[xt.Strategy(xt.Teapot(1, mode='thick'))])
     line.build_tracker(_context=test_context)
     assert line['e0..0'].parent_name == 'e0'
-    assert line['e0..0']._parent is line.element_dict['e0']
+    assert line['e0..0']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03,
                         _context=test_context)
@@ -647,7 +647,7 @@ def test_thick_slice_octupole(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..0'], xt.ThickSliceOctupole)
     assert line2['e0..0'].parent_name == 'e0'
-    assert line2['e0..0']._parent is line2.element_dict['e0']
+    assert line2['e0..0']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -669,7 +669,7 @@ def test_thick_slice_solenoid(test_context):
         slicing_strategies=[xt.Strategy(xt.Teapot(10, mode='thick'))])
     line.build_tracker(_context=test_context)
     assert line['e0..0'].parent_name == 'e0'
-    assert line['e0..0']._parent is line.element_dict['e0']
+    assert line['e0..0']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03,
                         _context=test_context)
@@ -695,7 +695,7 @@ def test_thick_slice_solenoid(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..0'], xt.ThickSliceUniformSolenoid)
     assert line2['e0..0'].parent_name == 'e0'
-    assert line2['e0..0']._parent is line2.element_dict['e0']
+    assert line2['e0..0']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -844,13 +844,13 @@ def test_thin_slice_bend_with_multipoles(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..995'].parent_name == 'e0'
-    assert line['e0..995']._parent is line.element_dict['e0']
+    assert line['e0..995']._parent is line.get('e0')
     assert line['drift_e0..995'].parent_name == 'e0'
-    assert line['drift_e0..995']._parent is line.element_dict['e0']
+    assert line['drift_e0..995']._parent is line.get('e0')
     assert line['e0..entry_map'].parent_name == 'e0'
-    assert line['e0..entry_map']._parent is line.element_dict['e0']
+    assert line['e0..entry_map']._parent is line.get('e0')
     assert line['e0..exit_map'].parent_name == 'e0'
-    assert line['e0..exit_map']._parent is line.element_dict['e0']
+    assert line['e0..exit_map']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -882,16 +882,16 @@ def test_thin_slice_bend_with_multipoles(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..995'], xt.ThinSliceBend)
     assert line2['e0..995'].parent_name == 'e0'
-    assert line2['e0..995']._parent is line2.element_dict['e0']
+    assert line2['e0..995']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..995'], xt.DriftSliceBend)
     assert line2['drift_e0..995'].parent_name == 'e0'
-    assert line2['drift_e0..995']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..995']._parent is line2.get('e0')
     assert isinstance(line2['e0..entry_map'], xt.ThinSliceBendEntry)
     assert line2['e0..entry_map'].parent_name == 'e0'
-    assert line2['e0..entry_map']._parent is line2.element_dict['e0']
+    assert line2['e0..entry_map']._parent is line2.get('e0')
     assert isinstance(line2['e0..exit_map'], xt.ThinSliceBendExit)
     assert line2['e0..exit_map'].parent_name == 'e0'
-    assert line2['e0..exit_map']._parent is line2.element_dict['e0']
+    assert line2['e0..exit_map']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -950,11 +950,11 @@ def test_thick_slice_bend_with_multipoles(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..995'].parent_name == 'e0'
-    assert line['e0..995']._parent is line.element_dict['e0']
+    assert line['e0..995']._parent is line.get('e0')
     assert line['e0..entry_map'].parent_name == 'e0'
-    assert line['e0..entry_map']._parent is line.element_dict['e0']
+    assert line['e0..entry_map']._parent is line.get('e0')
     assert line['e0..exit_map'].parent_name == 'e0'
-    assert line['e0..exit_map']._parent is line.element_dict['e0']
+    assert line['e0..exit_map']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -984,13 +984,13 @@ def test_thick_slice_bend_with_multipoles(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..995'], xt.ThickSliceBend)
     assert line2['e0..995'].parent_name == 'e0'
-    assert line2['e0..995']._parent is line2.element_dict['e0']
+    assert line2['e0..995']._parent is line2.get('e0')
     assert isinstance(line2['e0..entry_map'], xt.ThinSliceBendEntry)
     assert line2['e0..entry_map'].parent_name == 'e0'
-    assert line2['e0..entry_map']._parent is line2.element_dict['e0']
+    assert line2['e0..entry_map']._parent is line2.get('e0')
     assert isinstance(line2['e0..exit_map'], xt.ThinSliceBendExit)
     assert line2['e0..exit_map'].parent_name == 'e0'
-    assert line2['e0..exit_map']._parent is line2.element_dict['e0']
+    assert line2['e0..exit_map']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -1025,13 +1025,13 @@ def test_thin_slice_bend_with_multipoles_bend_off(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..5'].parent_name == 'e0'
-    assert line['e0..5']._parent is line.element_dict['e0']
+    assert line['e0..5']._parent is line.get('e0')
     assert line['drift_e0..5'].parent_name == 'e0'
-    assert line['drift_e0..5']._parent is line.element_dict['e0']
+    assert line['drift_e0..5']._parent is line.get('e0')
     assert line['e0..entry_map'].parent_name == 'e0'
-    assert line['e0..entry_map']._parent is line.element_dict['e0']
+    assert line['e0..entry_map']._parent is line.get('e0')
     assert line['e0..exit_map'].parent_name == 'e0'
-    assert line['e0..exit_map']._parent is line.element_dict['e0']
+    assert line['e0..exit_map']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -1063,16 +1063,16 @@ def test_thin_slice_bend_with_multipoles_bend_off(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..5'], xt.ThinSliceBend)
     assert line2['e0..5'].parent_name == 'e0'
-    assert line2['e0..5']._parent is line2.element_dict['e0']
+    assert line2['e0..5']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..5'], xt.DriftSliceBend)
     assert line2['drift_e0..5'].parent_name == 'e0'
-    assert line2['drift_e0..5']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..5']._parent is line2.get('e0')
     assert isinstance(line2['e0..entry_map'], xt.ThinSliceBendEntry)
     assert line2['e0..entry_map'].parent_name == 'e0'
-    assert line2['e0..entry_map']._parent is line2.element_dict['e0']
+    assert line2['e0..entry_map']._parent is line2.get('e0')
     assert isinstance(line2['e0..exit_map'], xt.ThinSliceBendExit)
     assert line2['e0..exit_map'].parent_name == 'e0'
-    assert line2['e0..exit_map']._parent is line2.element_dict['e0']
+    assert line2['e0..exit_map']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -1129,7 +1129,7 @@ def test_thick_slice_quad_with_multipoles(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..5'].parent_name == 'e0'
-    assert line['e0..5']._parent is line.element_dict['e0']
+    assert line['e0..5']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -1155,7 +1155,7 @@ def test_thick_slice_quad_with_multipoles(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..5'], xt.ThickSliceQuadrupole)
     assert line2['e0..5'].parent_name == 'e0'
-    assert line2['e0..5']._parent is line2.element_dict['e0']
+    assert line2['e0..5']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -1184,9 +1184,9 @@ def test_thin_slice_quad_with_multipoles(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..5'].parent_name == 'e0'
-    assert line['e0..5']._parent is line.element_dict['e0']
+    assert line['e0..5']._parent is line.get('e0')
     assert line['drift_e0..5'].parent_name == 'e0'
-    assert line['drift_e0..5']._parent is line.element_dict['e0']
+    assert line['drift_e0..5']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -1214,10 +1214,10 @@ def test_thin_slice_quad_with_multipoles(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..5'], xt.ThinSliceQuadrupole)
     assert line2['e0..5'].parent_name == 'e0'
-    assert line2['e0..5']._parent is line2.element_dict['e0']
+    assert line2['e0..5']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..5'], xt.DriftSliceQuadrupole)
     assert line2['drift_e0..5'].parent_name == 'e0'
-    assert line2['drift_e0..5']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..5']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -1272,9 +1272,9 @@ def test_thin_slice_quad_with_multipoles_quad_off(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..5'].parent_name == 'e0'
-    assert line['e0..5']._parent is line.element_dict['e0']
+    assert line['e0..5']._parent is line.get('e0')
     assert line['drift_e0..5'].parent_name == 'e0'
-    assert line['drift_e0..5']._parent is line.element_dict['e0']
+    assert line['drift_e0..5']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.5, y=0.3, py=0.3, delta=0.1
                       ,_context=test_context)
@@ -1302,10 +1302,10 @@ def test_thin_slice_quad_with_multipoles_quad_off(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..5'], xt.ThinSliceQuadrupole)
     assert line2['e0..5'].parent_name == 'e0'
-    assert line2['e0..5']._parent is line2.element_dict['e0']
+    assert line2['e0..5']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..5'], xt.DriftSliceQuadrupole)
     assert line2['drift_e0..5'].parent_name == 'e0'
-    assert line2['drift_e0..5']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..5']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -1360,7 +1360,7 @@ def test_thick_slice_sextupole_with_multipoles(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..1'].parent_name == 'e0'
-    assert line['e0..1']._parent is line.element_dict['e0']
+    assert line['e0..1']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -1386,7 +1386,7 @@ def test_thick_slice_sextupole_with_multipoles(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..0'], xt.ThickSliceSextupole)
     assert line2['e0..1'].parent_name == 'e0'
-    assert line2['e0..1']._parent is line2.element_dict['e0']
+    assert line2['e0..1']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -1414,9 +1414,9 @@ def test_thin_slice_sextupole_with_multipoles(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..1'].parent_name == 'e0'
-    assert line['e0..1']._parent is line.element_dict['e0']
+    assert line['e0..1']._parent is line.get('e0')
     assert line['drift_e0..1'].parent_name == 'e0'
-    assert line['drift_e0..1']._parent is line.element_dict['e0']
+    assert line['drift_e0..1']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -1444,10 +1444,10 @@ def test_thin_slice_sextupole_with_multipoles(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..1'], xt.ThinSliceSextupole)
     assert line2['e0..1'].parent_name == 'e0'
-    assert line2['e0..1']._parent is line2.element_dict['e0']
+    assert line2['e0..1']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..1'], xt.DriftSliceSextupole)
     assert line2['drift_e0..1'].parent_name == 'e0'
-    assert line2['drift_e0..1']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..1']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -1501,7 +1501,7 @@ def test_thick_slice_octupole_with_multipoles(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..1'].parent_name == 'e0'
-    assert line['e0..1']._parent is line.element_dict['e0']
+    assert line['e0..1']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -1527,7 +1527,7 @@ def test_thick_slice_octupole_with_multipoles(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..0'], xt.ThickSliceOctupole)
     assert line2['e0..1'].parent_name == 'e0'
-    assert line2['e0..1']._parent is line2.element_dict['e0']
+    assert line2['e0..1']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
@@ -1556,9 +1556,9 @@ def test_thin_slice_octupole_with_multipoles(test_context):
     line.build_tracker(_context=test_context)
     line._line_before_slicing.build_tracker(_context=test_context)
     assert line['e0..1'].parent_name == 'e0'
-    assert line['e0..1']._parent is line.element_dict['e0']
+    assert line['e0..1']._parent is line.get('e0')
     assert line['drift_e0..1'].parent_name == 'e0'
-    assert line['drift_e0..1']._parent is line.element_dict['e0']
+    assert line['drift_e0..1']._parent is line.get('e0')
 
     p0 = xt.Particles(p0c=10e9, x=0.1, px=0.2, y=0.3, py=0.4, delta=0.03
                       ,_context=test_context)
@@ -1586,10 +1586,10 @@ def test_thin_slice_octupole_with_multipoles(test_context):
     line2.build_tracker(_context=test_context)
     assert isinstance(line2['e0..1'], xt.ThinSliceOctupole)
     assert line2['e0..1'].parent_name == 'e0'
-    assert line2['e0..1']._parent is line2.element_dict['e0']
+    assert line2['e0..1']._parent is line2.get('e0')
     assert isinstance(line2['drift_e0..1'], xt.DriftSliceOctupole)
     assert line2['drift_e0..1'].parent_name == 'e0'
-    assert line2['drift_e0..1']._parent is line2.element_dict['e0']
+    assert line2['drift_e0..1']._parent is line2.get('e0')
 
     line.track(p_slice, backtrack=True)
 
