@@ -6,16 +6,19 @@
 #ifndef XTRACK_FASTQUADRUPOLE_H
 #define XTRACK_FASTQUADRUPOLE_H
 
-/*gpufun*/
+#include <headers/track.h>
+
+
+GPUFUN
 void SimpleThinQuadrupole_track_local_particle(SimpleThinQuadrupoleData el, LocalParticle* part0){
 
     double knl1 = SimpleThinQuadrupoleData_get_knl(el, 1);
 
-    #ifdef XSUITE_BACKTRACK
+    if (LocalParticle_check_track_flag(part0, XS_FLAG_BACKTRACK)) {
         knl1 = -knl1;
-    #endif
+    }
 
-    //start_per_particle_block (part0->part)
+    START_PER_PARTICLE_BLOCK(part0, part);
         double const x = LocalParticle_get_x(part);
         double const y = LocalParticle_get_y(part);
         double const chi = LocalParticle_get_chi(part);
@@ -26,7 +29,7 @@ void SimpleThinQuadrupole_track_local_particle(SimpleThinQuadrupoleData el, Loca
 
         LocalParticle_add_to_px(part, dpx);
         LocalParticle_add_to_py(part, dpy);
-    //end_per_particle_block
+    END_PER_PARTICLE_BLOCK;
 }
 
 #endif
