@@ -173,6 +173,7 @@ void track_magnet_body_single_particle(
     }
     else{
 
+        
     // START GENERATED INTEGRATION CODE
 
     if (integrator == 1){ // TEAPOT
@@ -271,6 +272,7 @@ void track_magnet_body_single_particle(
 
 }
 
+
 GPUFUN
 void track_magnet_particles(
     double const weight,
@@ -304,7 +306,10 @@ void track_magnet_particles(
     double x0_solenoid,
     double y0_solenoid,
     int64_t rbend_model, // -1: not used, 0: auto, 1: curved body, 2: straight body
+    int64_t rbend_compensate_sagitta,
     double rbend_shift,
+    double rbend_angle_diff,
+    double length_straight,
     int64_t body_active,
     int64_t edge_entry_active,
     int64_t edge_exit_active,
@@ -323,48 +328,67 @@ void track_magnet_particles(
     double factor_knl_ksl = 1.0;
 
     // Used only for rbend-straight-body
-    double rbend_half_angle = 0.;
-    double cos_rbha = 0.;
-    double sin_rbha = 0.;
+    double theta_in = 0.;
+    double theta_out = 0.;
+    double cos_theta_in = 1.;
+    double sin_theta_in = 0.;
+    double cos_theta_out = 1.;
+    double sin_theta_out = 0.;
     double length_curved = 0.;
-    double dx_rb = 0.;
-
+    double x0_mid = 0.;
+    double x0_in = 0.;
+    double x0_out = 0.;
 
     if (rbend_model == 0){
         // auto mode, curved body
         rbend_model = 1;
     }
 
+    double const angle = h * length;
     if (rbend_model == 1){
         // curved body
-        double const angle = h * length;
-        edge_entry_angle += angle / 2.0;
-        edge_exit_angle += angle / 2.0;
+        edge_entry_angle += (angle - rbend_angle_diff) / 2.0;
+        edge_exit_angle += (angle + rbend_angle_diff) / 2.0;
     }
     else if (rbend_model == 2){
         // straight body
-        double sinc_rbha;
-        rbend_half_angle = h * length / 2;
-        if (fabs(rbend_half_angle) > 1e-10){
-            sin_rbha = sin(rbend_half_angle);
-            cos_rbha = cos(rbend_half_angle);
-            sinc_rbha = sin_rbha / rbend_half_angle;
+        theta_in = (angle - rbend_angle_diff) / 2.0;
+        if (fabs(theta_in) > 1e-10){
+            sin_theta_in = sin(theta_in);
+            cos_theta_in = cos(theta_in);
         }
-        else {
-            sinc_rbha = 1.;
-            cos_rbha = 1.;
+        theta_out = (angle + rbend_angle_diff) / 2.0;
+        if (fabs(theta_out) > 1e-10){
+            sin_theta_out = sin(theta_out);
+            cos_theta_out = cos(theta_out);
         }
+
         length_curved = length;
-        length = length_curved * sinc_rbha;
-        if (fabs(rbend_half_angle) > 1e-10){
+        length = length_straight;
+
+        x0_mid -= rbend_shift;
+
+        if (rbend_compensate_sagitta && fabs(angle) > 1e-10){
             // shift by half the sagitta
-            dx_rb = 0.5 / h * (1 - cos_rbha) + rbend_shift;
-        };
+            double cos_rbha = cos(angle / 2.);
+            x0_mid += 0.5 / h * (1 - cos_rbha);
+        }
+
+        x0_in = x0_mid;
+        x0_out = x0_mid;
+        if (fabs(angle) > 1e-10){
+            double const px0_in = sin(theta_in);
+            double const px0_mid = px0_in - h * length_straight / 2;
+            double const sqrt_mid = sqrt(1 - px0_mid * px0_mid);
+            x0_in -= 1/h *(sqrt_mid - cos_theta_in);
+            x0_out += 1/h * (cos_theta_out - sqrt_mid);
+        }
+        ;
         h = 0; // treat magnet as straight
         // We are entering the fringe with an angle, the linear fringe
         // needs to come from the  expansion around the right angle
-        edge_entry_angle_fdown += rbend_half_angle;
-        edge_exit_angle_fdown += rbend_half_angle;
+        edge_entry_angle_fdown += theta_in;
+        edge_exit_angle_fdown += theta_out;
     }
 
     double core_length, core_length_curved,factor_knl_ksl_body,
@@ -383,8 +407,14 @@ void track_magnet_particles(
         VSWAP(edge_entry_angle_fdown, edge_exit_angle_fdown);
         VSWAP(edge_entry_fint, edge_exit_fint);
         VSWAP(edge_entry_hgap, edge_exit_hgap);
-        rbend_half_angle = -rbend_half_angle;
-        sin_rbha = -sin_rbha;
+        VSWAP(theta_in, theta_out);
+        VSWAP(cos_theta_in, cos_theta_out);
+        VSWAP(sin_theta_in, sin_theta_out);
+        theta_in = -theta_in;
+        theta_out = -theta_out;
+        sin_theta_in = -sin_theta_in;
+        sin_theta_out = -sin_theta_out;
+        VSWAP(x0_in, x0_out);
     }
     else {
         core_length = length * weight;
@@ -430,8 +460,9 @@ void track_magnet_particles(
         if (rbend_model == 2){
             // straight body --> curvature in the edges
             START_PER_PARTICLE_BLOCK(part0, part);
-                YRotation_single_particle(part, -sin_rbha, cos_rbha, -sin_rbha/cos_rbha);
-                LocalParticle_add_to_x(part, -dx_rb);
+                YRotation_single_particle(part, -sin_theta_in, cos_theta_in,
+                                          -sin_theta_in/cos_theta_in);
+                LocalParticle_add_to_x(part, x0_in);
             END_PER_PARTICLE_BLOCK;
         }
 
@@ -581,8 +612,9 @@ void track_magnet_particles(
         if (rbend_model == 2){
             // straight body --> curvature in the edges
             START_PER_PARTICLE_BLOCK(part0, part);
-                LocalParticle_add_to_x(part, dx_rb); // shift by half sagitta
-                YRotation_single_particle(part, -sin_rbha, cos_rbha, -sin_rbha/cos_rbha);
+                LocalParticle_add_to_x(part, -x0_out); // shift by half sagitta
+                YRotation_single_particle(part, -sin_theta_out, cos_theta_out,
+                    -sin_theta_out/cos_theta_out);
             END_PER_PARTICLE_BLOCK;
         }
     }
