@@ -26,6 +26,19 @@ void XRotation_single_particle(LocalParticle* part, double sin_angle, double cos
     double x_hat = x + tan_angle*y*px/(pz*ptt);
     double t_hat = t - tan_angle*y*(1.0/beta0+pt)/(pz*ptt);
 
+    /* Spin tracking is disabled by the synrad compile flag */
+    #ifndef XTRACK_MULTIPOLE_NO_SYNRAD
+        // Rotate spin
+        double const spin_y_0 = LocalParticle_get_spin_y(part);
+        double const spin_z_0 = LocalParticle_get_spin_z(part);
+        if ((spin_y_0 != 0) || (spin_z_0 != 0)){
+            double const spin_y_1 = cos_angle*spin_y_0 + sin_angle*spin_z_0;
+            double const spin_z_1 = -sin_angle*spin_y_0 + cos_angle*spin_z_0;
+            LocalParticle_set_spin_y(part, spin_y_1);
+            LocalParticle_set_spin_z(part, spin_z_1);
+        }
+    #endif
+
     LocalParticle_set_x(part, x_hat);
     LocalParticle_set_py(part, py_hat);
     LocalParticle_set_y(part, y_hat);
