@@ -20,7 +20,7 @@ EXTRA_PARAMS = {
 
 TRANSLATE_PARAMS = {
     "l": "length",
-    "lrad": "length",
+    "lrad": "lrad",
     "tilt": "rot_s_rad",
     "from": "from_",
     "e1": "edge_entry_angle",
@@ -64,8 +64,6 @@ def _warn(msg):
 
 def get_params(params, parent):
     params = params.copy()
-    if parent in {'placeholder', 'instrument'}:
-        _ = params.pop('lrad', None)
 
     def _normalise_single(param):
         lower = param.lower()
@@ -93,6 +91,8 @@ class MadxLoader:
             self,
             env: xt.Environment = None,
             default_to_zero: bool = False,
+            s_tol: float = 1e-9,
+            _rbend_correct_k0: bool = False,
     ):
         self._madx_elem_hierarchy: Dict[str, List[str]] = {}
         self._both_direction_elements: Set[str] = set()
@@ -102,6 +102,8 @@ class MadxLoader:
         self.env = env or xt.Environment()
         self.env.default_to_zero = default_to_zero
         self.builders = {}
+        self.s_tol = s_tol
+        self._rbend_correct_k0 = _rbend_correct_k0
 
         self._init_environment()
 
@@ -124,7 +126,9 @@ class MadxLoader:
         self._new_builtin("monitor", "Drift")
         self._new_builtin("hmonitor", "Drift")
         self._new_builtin("vmonitor", "Drift")
+        self._new_builtin("imonitor", "Drift")
         self._new_builtin("placeholder", "Drift")
+        self._new_builtin("wire", "Drift")
         self._new_builtin("sbend", "Bend")
         self._new_builtin("rbend", "RBend")
         self._new_builtin("quadrupole", "Quadrupole")
@@ -139,9 +143,7 @@ class MadxLoader:
         self._new_builtin("yrotation", "YRotation")
         self._new_builtin("srotation", "SRotation")
         self._new_builtin("translation", "XYShift")
-
-        for mad_apertype in _APERTURE_TYPES:
-            self._new_builtin(mad_apertype, 'Marker')
+        self._new_builtin("dipedge", "DipoleEdge")
 
     def load_file(self, file):
         """Load a MAD-X file and generate/update the environment."""
@@ -167,6 +169,42 @@ class MadxLoader:
             self._parse_parameters(parsed_dict["parameters"])
             self.builders.update(builders)
 
+        # Handle variables obtained from arrow operations
+        for var_name in self.env._xdeps_vref._owner.keys():
+            if var_name.startswith('_length__'):
+                elem_name = var_name[len('_length__') :]
+                self.env.vars[var_name] = self.env.ref[elem_name].length
+
+        # Handle edge angle feed-downs
+        for ename, elem in self.env._xdeps_eref._owner.items():
+            if isinstance(elem, (xt.RBend, xt.Bend)):
+                if elem.k0_from_h:
+                    continue
+                # I know it came from madx, so the expression is on angle not h
+                aa = self.env.ref[ename].angle._expr or float(self.env.ref[ename].angle._value)
+                kk0 = self.env.ref[ename].k0._expr or float(self.env.ref[ename].k0._value)
+                if isinstance(elem, xt.RBend):
+                    lstraight = self.env.ref[ename].length_straight._expr or self.env.ref[ename].length_straight._value
+                    lcurv = lstraight / self.env._xdeps_fref.sinc(aa / 2)
+                else:
+                    lcurv = self.env.ref[ename].length._expr or self.env.ref[ename].length._value
+                angle_fdown = 0.5 * (kk0 * lcurv - aa)
+                self.env.ref[ename].edge_entry_angle_fdown = angle_fdown
+                self.env.ref[ename].edge_exit_angle_fdown = angle_fdown
+
+        if self._rbend_correct_k0:
+            # needed for sequences (e.g. LHC) defined with rbarc=False in MAD-X
+            sinc = self.env.functions['sinc']
+            tt_rbend = self.env.elements.get_table().rows.match('RBend', 'element_type')
+            for nn in tt_rbend.name:
+                ee_ref = self.env.ref[nn]
+                ee = self.env.get(nn)
+                if ee.k0_from_h:
+                    continue
+                k0 = ee_ref.k0._expr or float(ee_ref.k0._value)
+                angle = ee_ref.angle._expr or float(ee_ref.angle._value)
+                self.env[nn].k0 = k0 * sinc(angle / 2)
+
     def _parse_elements(self, elements: Dict[str, ElementType]):
         for name, el_params in elements.items():
             parent = el_params.pop('parent')
@@ -194,7 +232,7 @@ class MadxLoader:
                 length = params.get('l', None)
                 builder = self.env.new_line(name=name, refer=refer,
                                                length=length,
-                                               s_tol=1e-6,
+                                               s_tol=self.s_tol,
                                                compose=True)
                 self._parse_components(builder, params.pop('elements'))
             elif line_type == 'line':
@@ -314,16 +352,7 @@ class MadxLoader:
 
         if (extras := el_params.pop('extra', None)):
             _warn(f'Ignoring extra parameters {extras} for element `{name}`!')
-        # element = self.env[name]
 
-        # length = self._element_length(name, el_params)
-        # is_not_thick = isinstance(element, BeamElement) and not element.isthick
-        # if is_not_thick and length and not isinstance(element, xt.Marker):
-        #     # Handle the thin elements that have a length in MAD-X: sandwich
-        #     line = self._make_thick_sandwich(name, length)
-        #     builder.place(line, **el_params)
-        # else:
-        #     builder.place(name, **el_params)
         builder.place(name, **el_params)
 
     def _clone_element(self, name, parent, builder, el_params):
@@ -331,23 +360,6 @@ class MadxLoader:
 
         Here `parent` is not None.
         """
-        # length = self._element_length(name, el_params)
-        # element = self.env[parent]
-        # is_not_thick = isinstance(element, BeamElement) and not element.isthick
-
-        # if is_not_thick and length and not isinstance(element, xt.Marker):
-        #     # Handle the thin elements that have a length in MAD-X: sandwich
-        #     at, from_ = el_params.pop('at', None), el_params.pop('from_', None)
-        #     if name not in self.env.element_dict:
-        #         make_drifts = True
-        #         self.env.new(name, parent, **el_params)
-        #     else:
-        #         make_drifts = False
-        #         _warn(f'Element `{name}` already exists, this definition '
-        #               f'will be ignored (for compatibility with MAD-X)')
-        #     name = self._make_thick_sandwich(name, length, make_drifts)
-        #     builder.place(name, at=at, from_=from_)
-        # elif name == parent:
         if name == parent:
             # This happens when the element is cloned with the same name, which
             # is allowed inside MAD-X sequences, e.g.: `el: el, at = 42;`.
@@ -363,26 +375,6 @@ class MadxLoader:
             # is retained: we do not simulate this behaviour here.
             builder.new(name, parent, force=True, **el_params)
 
-    def _element_length(self, name, el_params):
-        """Given the definition and params of the element, return its length."""
-        length = el_params.get('length', self._parameter_cache[name].get('length', 0))
-        THIN_ELEMENTS = {'vkicker', 'hkicker', 'kicker', 'tkicker', 'multipole'}
-        if self._mad_base_type(name) in THIN_ELEMENTS:
-            # Workaround for the elements that are thin despite having a
-            # ``length`` parameter.
-            length = 0
-
-        return length
-
-    # def _make_thick_sandwich(self, name, length, make_drifts=True):
-    #     """Make a sandwich of two drifts around the element."""
-    #     drift_name = f'{name}_drift'
-    #     if make_drifts:
-    #         self.env.new(drift_name + '_0', 'Drift', length=length / 2)
-    #         self.env.new(drift_name + '_1', 'Drift', length=length / 2)
-    #     line = self.env.new_line([drift_name + '_0', name, drift_name + '_1'])
-    #     return line
-
     def _set_element(self, name, builder, **kwargs):
         self._parameter_cache[name].update(kwargs)
 
@@ -406,8 +398,19 @@ class MadxLoader:
     def _convert_element_params(self, name, params):
         parent_name = self._mad_base_type(name)
 
+        if parent_name in {'multipole', 'hkicker', 'vkicker', 'kicker', 'tkicker'}:
+            # Elements using 'lrad'
+            if 'length' not in params and 'lrad' in params:
+                params['length'] = params.pop('lrad', None)
+        else:
+            # Elements not using 'lrad'
+            params.pop('lrad', None)
+
         if parent_name in {'sbend', 'rbend'}:
-            length = params.get('length', 0)
+            if 'k1s' in params:
+                raise ValueError(
+                    f'Cannot set `k1s` for element `{name}`: use `ksl` instead.'
+                )
 
             if parent_name == 'rbend':
                 if 'length' in params:
@@ -421,10 +424,6 @@ class MadxLoader:
             else:
                 params['k0_from_h'] = False
 
-            if (k2 := params.pop('k2', None)) and length:
-                params['knl'] = [0, 0, k2 * length]
-            if (k1s := params.pop('k1s', None)) and length:
-                params['ksl'] = [0, k1s * length]
             if (hgap := params.pop('hgap', None)):
                 params['edge_entry_hgap'] = hgap
                 params['edge_exit_hgap'] = hgap
@@ -449,8 +448,6 @@ class MadxLoader:
                 params['knl'] = knl
             if (ksl := params.pop('ksl', None)):
                 params['ksl'] = ksl
-            if params.pop('lrad', None):
-                _warn(f'Multipole `{name}` was specified with a length, ignoring!')
             for kk in list(params.keys()):
                 if kk.startswith('k') and kk.endswith('l'):
                     if kk == 'ksl' or kk == 'knl':
@@ -461,6 +458,16 @@ class MadxLoader:
                     if len(params['knl']) <= order:
                         params['knl'] += [0] * (order - len(params['knl']) + 1)
                     params['knl'][order] = params.pop(kk)
+            if angle := params.pop('angle', None):
+                params['hxl'] = angle
+            elif knl and len(knl) > 0:
+                params['hxl'] = knl[0]
+
+        elif parent_name == 'dipedge':
+            if edge_angle := params.pop('edge_entry_angle', None):
+                params['e1'] = edge_angle
+            if h := params.pop('h', None):
+                params['k'] = h
 
         elif parent_name == 'vkicker':
             if (kick := params.pop('kick', None)):
@@ -534,16 +541,29 @@ class MadxLoader:
             apertype = 'polygon'
             aperture = None
         else:
-            apertype = params.pop('apertype', None) or self._mad_base_type(name)
+            apertype = params.pop('apertype', 'circle')
             aperture = params.pop('aperture', None)
 
+        if 'apertype' not in self._parameter_cache[name]:
+            # Save the aperture type, as it might have been inferred (polygon, circle)
+            self._parameter_cache[name]['apertype'] = apertype
+
         if apertype not in _APERTURE_TYPES:
             raise ValueError(
                 f'The aperture type for the element `{name}` (inferred to be '
                 f'`{apertype}`) is not recognised.'
             )
 
-        x_offset, y_offset = params.pop('aper_offset', (0, 0))
+        if aperture is not None and not isinstance(aperture, list):
+            # Ensure if defined, aperture is a list
+            aperture = [aperture]
+
+        aper_offsets = params.pop('aper_offset', (0, 0))
+        if len(aper_offsets) == 1:
+            x_offset = aper_offsets[0]
+            y_offset = 0.
+        else:
+            x_offset, y_offset = aper_offsets
         if params.pop('aper_tol', None):
             _warn(f'Aperture tolerance (`{name}`) is not supported, ignoring.')
         aper_params = {
@@ -644,7 +664,8 @@ class MadxLoader:
         return self._mad_base_type(element_name) in _APERTURE_TYPES
 
 
-def load_madx_lattice(file=None, string=None, reverse_lines=None):
+def load_madx_lattice(file=None, string=None, reverse_lines=None, s_tol=1e-6,
+                      _rbend_correct_k0=False) -> xt.Environment:
 
     if file is not None and string is not None:
         raise ValueError('Only one of `file` or `string` can be provided!')
@@ -652,10 +673,13 @@ def load_madx_lattice(file=None, string=None, reverse_lines=None):
     if file is None and string is None:
         raise ValueError('Either `file` or `string` must be provided!')
 
-    loader = MadxLoader()
+    loader = MadxLoader(s_tol=s_tol, _rbend_correct_k0=_rbend_correct_k0)
 
     if file is not None:
-        loader.load_file(file)
+        if not isinstance(file, (tuple, list)):
+            file = [file]
+        for ff in file:
+            loader.load_file(ff)
     elif string is not None:
         loader.load_string(string)
     else:
