@@ -878,7 +878,7 @@ class OptimizeLine(xd.Optimize):
                     # If _end_point preceded by a marker, use the marker
                     if tt_at == '_end_point' and len(tw0.name) > 1:
                         nn_prev = tw0['name', -2]
-                        nn_env_prev = tw0['name_env', -2]
+                        nn_env_prev = tw0['env_name', -2]
                         if isinstance(this_line[nn_env_prev], xt.Marker):
                             tt_at= nn_prev
                 tt.tar = (tt_name, tt_at)
