@@ -9,7 +9,7 @@
 #ifndef XTRACK_BEAM_POSITION_MONITOR_H
 #define XTRACK_BEAM_POSITION_MONITOR_H
 
-#include <headers/track.h>
+#include "xtrack/headers/track.h"
 
 
 GPUFUN
