@@ -149,6 +149,15 @@ class TrackerData:
                 this_parent._movable = True
                 assert self._element_dict[nn]._parent._offset == self._element_dict[nn]._xobject._parent._offset
 
+    def __del__(self):
+        # Free the element ref data
+        if hasattr(self, '_element_ref_data'):
+            offset = self._element_ref_data._offset
+            buffer = self._element_ref_data._buffer
+            size = self._element_ref_data._size
+            del self._element_ref_data
+            buffer.free(offset, size)
+
     def common_buffer_for_elements(self):
         """If all `self.elements` elements are in the same buffer,
         returns said buffer, otherwise returns `None`."""
@@ -269,9 +278,9 @@ class TrackerData:
         self.line_element_classes = [cc._XoStruct for cc in self.line_element_classes]
         self.extra_element_classes = [cc._XoStruct for cc in self.extra_element_classes]
 
-        element_refs_cls = xt.tracker._element_ref_data_class_from_element_classes(
+        element_ref_cls = xt.tracker._element_ref_data_class_from_element_classes(
                                             kernel_element_classes)
-        self._element_ref_data = element_refs_cls._from_buffer(
+        self._element_ref_data = element_ref_cls._from_buffer(
             buffer=buffer,
             offset=offset,
         )
